{
	/* 	I took the original Smart Import script from Adobe and modified it to be much more robust and to fit our workflow.
		Spencer Tweed, SMP, Nov 2015. Tested on AE CS6, on Windows.
		v0.2 Got recursive imports working, recognizing image sequences and such.
		v1.0 Recursive importing of all files in a folder structure, with a first iteration GUI. No real features yet.
		v1.1 Added "skip existing"
		v1.2 Added "FPS" and reworked "Skip existing" a bit. Also fixed bug where all imports were set to "skip existing"
	*/
	
	function GetProjectChildItems (selectedFolder)
	{
		if (selectedFolder == null) {
			var myItems = app.project.items;
		} else {
			var myItems = selectedFolder.items;
		}

		//recursion to step through all sub-folders and grab their childrens
		var itemsNameArray = [null];
		function HideYourKids (myItems, itemsNameArray) {
			for (var i = 0; i < myItems.length; i++) {
				if (myItems[i+1] instanceof FootageItem) {
					//alert("got one: " + myItems[i+1].name);
					itemsNameArray[itemsNameArray.length] = myItems[i+1].name;
				}
				if (myItems[i+1] instanceof FolderItem) {
					HideYourKids(myItems[i+1].items, itemsNameArray);
				}
			}
			return itemsNameArray;
		}

		return HideYourKids(myItems, itemsNameArray);
	}

	function ST_SmartImport(targetFolder, skipExisting, FPS, selectedFolder)
	{
		if (targetFolder != null) {
			function importSafeWithError(importOptions, selectedFolder)
			{
				try {
					if (selectedFolder != null) {
						if (selectedFolder.typeName == "Folder") {
							if (skipExisting != true) {
								var importedFile = app.project.importFile(importOptions);
								importedFile.parentFolder = selectedFolder;
								importedFile.mainSource.conformFrameRate = FPS;
							} else {
								// skip existing. needs optomization
								//alert(selectedFolder.name);
								var itemsNameArray = GetProjectChildItems(selectedFolder);
								var importedFile = app.project.importFile(importOptions);

								// thingamajig to see if this is already in the project.
								var n= -1, N = itemsNameArray.length;
								while (++n<N && itemsNameArray[n]!==importedFile.name);
								var doesItemExist = n<N ? n: -1;
								if (doesItemExist != -1) {
									//already exists, lets get rid of it. Not fast, but effective.
									//print("Skipping: " + importedFile.name);
									importedFile.remove();
								} else {
									importedFile.parentFolder = selectedFolder;
									importedFile.mainSource.conformFrameRate = FPS;
								}
							}
						} else {
							alert("something didn't go as planned.\nI forgot what I was doing here...");
						}
					} else {
						//import into root, no folder selected.
						if (skipExisting != true) {
							importedFile = app.project.importFile(importOptions);
							importedFile.mainSource.conformFrameRate = FPS;	
						} else {
							// skip existing. needs optomization
							var itemsNameArray = GetProjectChildItems(selectedFolder);
							var importedFile = app.project.importFile(importOptions);
							//alert("Here: " + importedFile.name);

							// thingamajig to see if this is already in the project.
							//alert("Checking... " + importedFile.name);
							var n= -1, N = itemsNameArray.length;
							while (++n<N && itemsNameArray[n]!==importedFile.name);
							var doesItemExist = n<N ? n: -1;
							if (doesItemExist != -1) {
								//already exists, lets get rid of it. Not fast, but effective.
								//alert("Skipping: " + importedFile.name);
								importedFile.remove();
							} else {
								importedFile.mainSource.conformFrameRate = FPS;
							}
						}
					}

				} catch (error) {
					alert(error.toString() + importOptions.file.fsName, scriptName);
				}
			}
			
			
			function processFolder(theFolder, selectedFolder)
			{
				var files = theFolder.getFiles().sort();	//for some reason comes in not sorted, so lets sort that bitch!

				//prune the array for sequences
				var movieFileCheck = new RegExp("(mov|avi|mpg)$", "i");
				var searcher = new RegExp("\\d{4,}"); //Regex for seeing if we have at least 4 numbers in the file name
				for (var i = 0; i < files.length; i++) {
					// Go through the array and strip out all elements that are the same once their numbers have been removed with a regex
					if (i > 0) {
						currentResult = searcher.exec(files[i].name); //check if a sequence
						if (currentResult) { // it is a sequence
							// first parse out hte comparison strings - current item and item before
							var testNameBefore = files[i-1].name;

							//if we have a sequence before our current item, we need to delete the numbers.
							var beforeNum = searcher.exec(testNameBefore);
							if (beforeNum) {
								var testNameBefore = testNameBefore.replace(beforeNum[0], "");	//remove the numbers from the file before
							}

							var testNameCurrent = files[i].name.replace(currentResult[0], "");	//remove the numbers from the current frame

							//compare to the element before it and delete if the same!!
							if (testNameBefore == testNameCurrent) {
								files.splice(i, 1);
								i--;
							}
						}
					}
				}

				//so now we import everything in the array. We check if it is a sequence, and if so we import it as such. If it is a folder, we recurse.
				for (index in files) {
					if (files[index] instanceof File) {
						//check for sequence
						var sequenceStartFile = searcher.exec(files[index].name);
						if (sequenceStartFile) {
							//alert("Importing as sequence: " + files[index].name);
							var importOptions = new ImportOptions(files[index]);
							importOptions.sequence = true;
							importSafeWithError(importOptions, selectedFolder);
						} else {
							//guess it wasn't a sequence!
							//alert("Importing: " + files[index].name);
							try {
								var importOptions = new ImportOptions(files[index]);
								importSafeWithError(importOptions, selectedFolder);
							} catch (error) {
								// Ignore errors.
							}
						}
					}
					if (files[index] instanceof Folder) {
						processFolder(files[index], selectedFolder); //recursion
					}
				}
			}
			
			processFolder(targetFolder, selectedFolder);
		}
	}
	
	
	//---------------------------------------------------- BUILD THE UI ----------------------------------------------------\\
	var stSmartImportWin = new Window("palette", "ST Smart Importer");
		stSmartImportWin.orientation = "column";
		//stSmartImportWin.margins = [3,3,3,3];
		stSmartImportWin.border = [3,3,3,3];
		var groupOne = stSmartImportWin.add("group");
			groupOne.add("statictext", undefined, "Path:");
			var path = groupOne.add("edittext", [0,0,400,18]);
			groupOne.add("statictext", undefined, "FPS:");
			var FPS = groupOne.add("edittext", [0,0,35,18]);
				FPS.text = 29.97;
			
		var groupTwo = stSmartImportWin.add("group");
			//var optionsPanel = groupTwo.add("panel", undefined, "Options");
				groupTwo.orientation = "row";
				var smartSettingsChk = groupTwo.add("checkbox", undefined, "Smart settings");
				var folderStructureChk = groupTwo.add("checkbox", undefined, "Maintain folder structure");
				var skipExistingChk = groupTwo.add("checkbox", undefined, "Skip existing");

		var groupThree = stSmartImportWin.add("group");
			var chooseFolderBtn = groupThree.add("button", undefined, "Choose folder");
			var importBtn = groupThree.add("button", undefined, "Import!");

	//skipExistingChk.value = true;
	//skipExistingChk.enabled = false;
	folderStructureChk.enabled = false;
	smartSettingsChk.enabled = false;

	//-------| Event listeners
	chooseFolderBtn.onClick = function () {
		var targetFolder = File.saveDialog("Import items from folder...");
		targetFolder = targetFolder.toString().split("/");
		targetFolder.pop();
		targetFolder = targetFolder.join("/");
		path.text = Folder.decode(targetFolder);
	}

	importBtn.onClick = function () {
		var targetFolder = new Folder(Folder.encode(path.text));
		ST_SmartImport(targetFolder, skipExistingChk.value, FPS.text, app.project.activeItem);
		stSmartImportWin.close();
	}
	

	stSmartImportWin.center();
	stSmartImportWin.show();
}